@@ -1,6 +1,7 @@
 import singer
 import unittest
-
+import datetime
+import dateutil
 
 class TestSinger(unittest.TestCase):
     def test_parse_message_record_good(self):
@@ -10,6 +11,30 @@ class TestSinger(unittest.TestCase):
             message,
             singer.RecordMessage(record={'name': 'foo'}, stream='users'))
 
+    def test_parse_message_record_with_version_good(self):
+        message = singer.parse_message(
+            '{"type": "RECORD", "record": {"name": "foo"}, "stream": "users", "version": 2}')
+        self.assertEqual(
+            message,
+            singer.RecordMessage(record={'name': 'foo'}, stream='users', version=2))
+
+    def test_parse_message_record_naive_extraction_time(self):
+        with self.assertRaisesRegex(ValueError, "must be either None or an aware datetime"):
+            message = singer.parse_message(
+                '{"type": "RECORD", "record": {"name": "foo"}, "stream": "users", "version": 2, "time_extracted": "1970-01-02T00:00:00"}')
+
+    def test_parse_message_record_aware_extraction_time(self):
+        message = singer.parse_message(
+            '{"type": "RECORD", "record": {"name": "foo"}, "stream": "users", "version": 2, "time_extracted": "1970-01-02T00:00:00.000Z"}')
+        expected = singer.RecordMessage(
+            record={'name': 'foo'},
+            stream='users',
+            version=2,
+            time_extracted=dateutil.parser.parse("1970-01-02T00:00:00.000Z"))
+        print(message)
+        print(expected)
+        self.assertEqual(message, expected)
+
     def test_parse_message_record_missing_record(self):
         with self.assertRaises(Exception):
             singer.parse_message('{"type": "RECORD", "stream": "users"}')
@@ -66,13 +91,13 @@ class TestSinger(unittest.TestCase):
                         'name': {'type': 'string'}}})
 
         state_message = singer.StateMessage(value={'seq': 1})
-
+        
         self.assertEqual(record_message,
-                         singer.parse_message(record_message.tojson()))
+                         singer.parse_message(singer.format_message(record_message)))
         self.assertEqual(schema_message,
-                         singer.parse_message(schema_message.tojson()))
+                         singer.parse_message(singer.format_message(schema_message)))
         self.assertEqual(state_message,
-                         singer.parse_message(state_message.tojson()))
+                         singer.parse_message(singer.format_message(state_message)))
 
     ## These three tests just confirm that writing doesn't throw
 
